;;;; Copyright (C) 1994 R. Dennis Cook and Sanford Weisberg.  The R-code is 
;;;; XLISP-STAT computer code to accompany the book \"Introduction to
;;;; Regression Graphics\", to be published by John Wiley, Inc.
;;;; Until publication of the book in 1994,
;;;; the program may be used by students at the University of
;;;; Minnesota, or by others with the written permission of the
;;;; copyright holders. 

(defun probplotdemo ()
   (let* ((menu (send menu-proto :new "ProbPlots"))
          (items '( "Probability plots"
                    "Quit"))
          (actions (list #'probplot-demo
                     #'(lambda () (send menu :remove)))))
   (apply #'send menu :append-items
          (mapcar #'(lambda (i a) (send menu-item-proto :new i :action a))
            items actions))
   (send menu :install)
   menu))

;;;
;;;  function to do a probability plot simulation
;;;

(defun probplot-demo ()
"This function is a probability plot simulator.  Via a dialog box,the user
selects a sample size, a sampling distribution, and a distribution to provide
population quantiles."
  (let* ((ans (let ((a (prob-plot-dialog)))
                   (if a a (return-from probplot-demo))))
         (n (select ans 0))
         (samp-name (case (select ans 1)
                          (0 "Normal")
                          (1 "Uniform")
                          (2 (format nil "ChiSq[~a]" (select ans 2)))
                          (3 (format nil "t[~a]" (select ans 2)))))
         (target-name (case (select ans 3)
                          (0 "Normal")
                          (1 "Uniform")
                          (2 (format nil "ChiSq[~a]" (select ans 4)))
                          (3 (format nil "t[~a]" (select ans 4)))))
         (target-dist  (select (list #'normal-quant
                            #'(lambda (p) p)
                            #'(lambda (p) (gamma-quant p (select ans 4)))
                            #'(lambda (p) (t-quant p (select ans 4))))
                                (select ans 3)))
         (samp-dist  (select (list #'normal-rand
                            #'uniform-rand
                            #'(lambda (n) (gamma-rand n (select ans 2)))
                            #'(lambda (n) (t-rand n (select ans 4))))
                            (select ans 1))) 
         (plot (plot-points (funcall target-dist (/ (1+ (iseq n)) (1+ n)))
                             (sort-data (funcall samp-dist n))
                             :show (showplot)
                             :title (format nil "Pop=~a; sample=~a"
                                            target-name samp-name)
                             :variable-labels (list
                              (format nil "Pop. (~a) Quantiles"
                                      target-name)
                              (format nil "Sample (~a) Quantiles"
                                      samp-name))))
         control)
        (send plot :plot-controls)
        (send plot :show-window)
        (setf control (send checkbox-control-proto :new
                            :location (send plot :locate-next-control)
                            :title "New sample"))
        (defmeth control :do-action (&rest args) (send plot :new-sample))
        (send plot :finish-next-frame 
              #'(lambda () (send plot :adjust-to-data)
                        (send plot :buffer-to-screen)))
        (send plot :start-next-frame
              #'(lambda () (send plot :start-buffering)))
        (send plot :add-overlay control)
        (send plot :redraw)
        (defmeth plot :new-sample ()
                 (send self :draw-next-frame '(1) (list
                      (sort-data (funcall samp-dist n)))))
  plot))

(defun prob-plot-dialog ()
"This function creates the special dialog for the probability plot simulator.
When ok is pushed, the function get-answer is executed, and this returns the
needed values.  Notice the use of with-input-from-string, that converts a
string to a number."
  (let* ((title (send text-item-proto :new "Probability plot simulator..."))
         (samp-size-name (send text-item-proto :new "Sample size"))
         (samp-dist-name (send text-item-proto :new 
                               "Sampling Distribution   "))
         (samp-dist (send choice-item-proto :new
                          '("Normal" "Uniform" "Chi-squared"
                                     "t-distribution") :value 0))
         (samp-df-name (send text-item-proto :new "D. F. ="))
         (samp-df (send edit-text-item-proto :new "3" :text-length 4))
         (target-dist-name (send text-item-proto :new 
                                 "Hypothesized Distribution"))
         (target-dist (send choice-item-proto :new
                          '("Normal" "Uniform" "Chi-squared"
                                     "t-distribution") :value 0))
         (target-df-name (send text-item-proto :new "D. F. ="))
         (target-df (send edit-text-item-proto :new "3" :text-length 4))
         (samp-size (send edit-text-item-proto :new "50" :text-length 4 ))
         (get-answer #'(lambda () 
                   (list (with-input-from-string 
                          (s (send samp-size :text)) (read s))
                         (send samp-dist :value)
                         (with-input-from-string
                          (s (send samp-df :text)) (read s))
                         (send target-dist :value)
                         (with-input-from-string
                          (s (send target-df :text)) (read s)))))
         (cancel (send modal-button-proto :new "Cancel"))
         (ok (send modal-button-proto :new "OK" :action 
                     #'(lambda () (funcall get-answer)))) 
         (dialog (send modal-dialog-proto :new
                           (list
                            title
                            (list samp-size-name samp-size)
                            (list samp-dist-name (list samp-dist
                                    (list samp-df-name samp-df)))
                            (list target-dist-name (list target-dist
                                    (list target-df-name target-df)))
                            (list ok cancel)) )))
        (send dialog :default-button ok)
        (send dialog :modal-dialog)))

(probplotdemo)
