(defun contour-dialog ()
(many-value-dialog "Type values for: mean1 mean2 sd1 sd2 rho n" 6 
    :length 40 :defaults '(0 0 1 1 .5 200)))

(defun r2-demo (&optional (numpts 30) (levels '(.1 .3 .5 .7 .9 .98)))
  (let* ((ans (contour-dialog))) 
   (if ans 
    (if (and (> (select ans 2) 0)
             (> (select ans 3) 0)
             (> (select ans 5) 0))
        (apply #'r2-norm (append ans (list numpts levels)))
        (block foo
         (message-dialog "sd1, sd2 and n must be positive")
         (r2-demo))))))


;bn-sample computes a sample of size n from a bivariate normal with 
;means mu1 and mu2, standard deviations s1 and s2 and correlation r.

(defun bn-sample (mu1 mu2 s1 s2 r n)
"Function args: mu1 mu2 s1 s2 r n
Returns a bivariate sample of size n from a normal distribution with mean
(mu1, mu2) variances s1 and s2, and covaraince r sqrt(s1*s2)."
   (let* ((z1 (normal-rand n))
          (z2 (normal-rand n))
          (x1 (+ mu1 (* s1 z1)))
          (fit (+ mu2 (* r s2 z1)))
          (x2 (+ fit (* (sqrt (- 1 (^ r 2))) s2 z2)))
         )
   (list x1 x2)))

(defun r2-norm (mu1 mu2 s1 s2 r n numpts levels &optional (dataset nil))
 (let* ((data (bn-sample mu1 mu2 s1 s2 r n)))
   (when data (make-dataset :data data :data-names '("X" "Y") :name "Normal"))))
         
(r2-demo)
