;;;; Copyright (C) 1994 R. Dennis Cook and Sanford Weisberg.  The R-code is 
;;;; XLISP-STAT computer code to accompany the book \"Introduction to
;;;; Regression Graphics\", to be published by John Wiley, Inc.
;;;; Until publication of the book in 1994,
;;;; the program may be used by students at the University of
;;;; Minnesota, or by others with the written permission of the
;;;; copyright holders. 

(defun 2d-demos ()
   (let* ((menu (send menu-proto :new "Demo:2D"))
          (items '( "Change Scales" "Birth Rates"
                    "Ozone data" "Small Ozone" "Brain Weight Data"
                    "More Birth Rates"
                    "Quit"))
          (actions (list #'cs #'birth1 #'ozone1 
								 #'(lambda () (ozone1 100))
								 #'bw1 #'birth2 
                     #'(lambda () (send menu :remove)))))
   (apply #'send menu :append-items
          (mapcar #'(lambda (i a) (send menu-item-proto :new i :action a))
            items actions))
   (send menu :install)
   menu))

(defun cs ()
   (let ((p (plot-points (iseq 1 60) 
              (+ (sin (rseq 1 40 60)) 
                 (* .1 (normal-rand 60)))
              :title "Can you find a pattern?"
              :variable-labels '("X" "Y") :show (showplot))))
        (send p :plot-controls)
        (send p :show-window)
        p))

(defun bw1 ()
   (let ((p (plot-points (select bw 0) (select bw 1) 
             :point-labels s-names
             :variable-labels '("BodyWt" "BrainWt") :show (showplot))))
       (send p :install-transform-control)
       (send p :show-window)))

(DEF BW (QUOTE ((3.385 0.48 1.35 464.983 36.328 27.66 14.831 1.04 4.19 0.425
0.101 0.92 1 0.005 0.06 3.5 2 1.7 2547.07 0.023 187.092 521.026 0.785 10 3.3
0.2 1.41 529.006 206.996 85.004 0.75 61.998 6654.18 3.5 6.8 34.998 4.05 0.12
0.023 0.01 1.4 250.01 2.5 55.501 100.003 52.159 10.55 0.55 59.997 3.6 4.288
0.28 0.075 0.122 0.048 192.001 3 160.004 0.9 1.62 0.104 4.235) (44.5 15.499
8.1 423.012 119.498 114.996 98.199 5.5 57.998 6.4 4 5.7 6.6 0.14 1 10.8 12.3
6.3 4603.17 0.3 419.012 654.977 3.5 114.996 25.6 5 17.5 680.005 406.019
324.992 12.3 1320.02 5711.86 3.9 179.003 56.003 17 1 0.4 0.25 12.5 489.997
12.1 175.002 156.993 440.011 179.504 2.4 80.996 21 39.201 1.9 1.2 3 0.33
180.008 25.001 169 2.6 11.4 2.5 50.4))))
  
(def s-names '("Arctic fox" "Owl monkey" "Beaver" "Cow" 
              "Gray wolf" "Goat" "Roe deer" "Guinea pig" 
              "Vervet" "Chinchilla" "Ground squirrel" 
              "Arctic ground squirrel" "African giant pouched rat"
              "Lesser short-tailed shrew" "Star-nosed mole"
              "Nine-banded armadillo" "Tree hyrax" 
              "N. American opossum" "Asian elephant" 
              "Big brown bat" "Donkey" "Horse" 
              "European hedgehog" "Patas monkey" "Cat" "Galago"
              "Genet" "Giraffe" "Gorilla" "Gray seal" 
              "Rock hyrax1"
              "Human" "African elephant" "Water opossum" 
              "Rhesus monkey" "Kangaroo" "Yellow-bellied marmot"
              "Golden hamster" "Mouse" "Little brown bat"
              "Slow loris" "Okapi" "Rabbit" "Sheep" "Jaguar"
              "Chimpanzee" "Baboon" "Desert hedgehog" 
              "Giant armadillo" "Rock hyrax2" "Raccoon" "Rat" 
              "E. American mole" "Mole rat" "Musk shrew" "Pig"
              "Echidna" "Brazilian tapir" "Tenrec" "Phalanger"
              "Tree shrew" "Red fox"))

(defun birth1 ()
   (defun source () (format t "
Monthly Birth rate = births per 1000 US population, 1940-47.  Date is coded as
a fraction of a year.  Only last 2 digits of the year are shown on 
the plot.~%"))

(let* ((birth-rate (* .01 (list 1890 1957 1925 1885 1896 1934 2036 2069 2060 
                                1922 1854 1852 1952 2011 2015 1971 1883 2070 
                                2221 2173 2105 1962 1951 1975 2092 2148 2114 
                                2013 1986 2088 2218 2312 2462 2455 2357 2309 
                                2398 2400 2331 2222 2156 2256 2352 2371 2356 
                                2211 2108 2069 2123 2147 2050 1977 1993 2134 
                                2275 2262 2194 2109 2114 2086 2089 2097 2036 
                                1957 1953 2039 2116 2134 2142 2023 1972 1942 
                                1931 1980 1977 1972 2017 2161 2468 2691 2890 
                                2913 2940 2870 2911 2832 2774 2568 2574 2641 
                                2691 2698 2701 2596 2503 2424)))
       (year (repeat (iseq 40 47) (repeat 12 8)))
       (month (repeat (/ (iseq 12) 12) 8))
       (names (repeat '("Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" 
                              "Sep" "Oct" "Nov" "Dec") 8))
       (p (plot-points (+ year month) birth-rate :variable-labels 
                      '("Year" "Monthly Birth Rate" )
                      :title "U.S. Birth Rates around WWII")))
  (source)
  (send p :plot-controls)
  (send p :point-label (iseq 96) names)
  p))

(defun birth2 ()
(defun source () (format t "
Birth rate = number of births per 1000 pop in US, 1956-1975.
Year
Month in year, coded in 1/12ths.~%"))

(let* ((birth-rate2 (* .1 (list 
240 240 235 234 228 229 219 213 206 191 178 175 165 172 175 179 157 150 144 144
246 243 241 239 233 235 224 216 211 195 183 178 169 176 181 180 160 150 146 146
244 241 236 239 231 233 223 214 206 194 183 177 167 173 178 176 155 151 144 145
225 236 229 228 223 224 215 209 200 185 179 167 171 171 172 169 149 144 142 144
222 238 232 225 219 221 213 209 202 187 176 173 171 169 172 163 153 143 143 145
243 248 237 236 226 228 215 214 212 194 182 178 176 176 181 168 152 147 144 146
260 261 244 248 246 240 230 225 221 202 187 184 182 186 191 174 157 154 156 155
269 262 248 252 257 248 240 229 219 204 193 186 184 187 191 178 163 158 159 155
271 267 263 258 262 250 241 235 226 209 198 189 186 188 198 183 165 157 163 156
258 258 252 244 242 237 229 222 214 194 188 180 179 181 186 172 156 149 156 148
248 246 241 236 236 229 221 211 205 189 183 174 177 178 184 165 154 146 148 142
247 243 240 236 238 226 220 211 203 186 181 173 179 182 189 162 156 146 148 146
)))
       (years (iseq 56 75)) 
       (month-names '("Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul"
                      "Aug" "Sep" "Oct" "Nov" "Dec"))
       (code-months (/ (iseq 12) 12)))
(source)
(make-reg :data (list birth-rate2 (repeat years 12)
                      (repeat code-months (repeat 20 12))
                      (mapcar #'(lambda (m y) (format nil "~a~a" m y))
                        (repeat month-names (repeat 20 12))
                        (repeat years 12)))
  :data-names '("BirthRate" "Year" "Month" "Names") :menu "Births")))

(defun ozone1 (&optional n)
  (def ozone '(
3 5 5 6 4 4 6 7 4 6 5 4 4 7 5 9 4 3 4 4 5 6 9 6 6 11 10 7 12 9 2 3 3 2 3 
3 4 6 8 6 4 3 7 11 13 6 5 4 4 6 10 15 23 17 7 2 3 3 4 6 7 7 6 3 2 8 12 12 
16 9 24 13 8 10 8 9 10 14 9 11 7 9 12 12 8 9 5 4 4 9 13 5 10 10 7 5 4 7 3 
4 7 11 15 22 17 7 10 19 18 12 6 9 19 21 29 16 11 2 12 16 22 20 27 33 25 31 
18 24 16 12 9 16 8 9 29 20 5 5 11 12 19 17 19 16 14 10 9 7 5 2 12 22 17 26 
27 14 11 23 26 21 15 20 15 18 26 19 13 30 26 15 16 16 19 23 28 34 33 24 17 
10 14 13 17 22 19 20 25 28 29 23 26 14 13 26 22 14 13 9 12 14 24 19 16 7 2 
4 6 12 9 15 17 13 20 22 24 26 32 33 27 38 23 19 19 15 28 10 14 26 17 3 14 29 
18 3 9 19 8 23 13 7 3 5 11 12 5 4 5 4 10 17 26 30 18 12 7 15 12 7 28 22 
18 14 24 10 14 9 12 7 7 6 13 5 3 7 8 10 12 6 5 20 14 16 5 3 5 1 5 4 11 6 8 
14 18 12 9 7 14 4 3 3 3 3 3 3 3 6 6 5 3 4 7 5 5 4 3 2 5 3 4 4 6 6 3 4 3 8 2 
3 5 1))
  (def temperature '(
40 45 54 35 45 55 41 44 54 51 51 54 58 61 64 67 52 54 54 58 69 51
53 59 64 63 63 62 63 54 36 31 30 36 42 37 41 46 51 55 41 41 49 45 55 
41 45 51 53 50 60 54 53 53 44 40 30 25 40 45 51 48 45 47 43 49 56
53 61 63 70 57 35 52 59 67 57 42 55 40 39 57 58 62 51 61 34 35 35
48 59 41 58 51 46 40 35 46 39 32 47 50 65 66 69 59 64 68 49 52 56 55 
66 74 76 70 67 41 61 68 69 74 74 80 84 78 80 76 74 66 66 69 62 67
80 76 65 65 63 62 68 68 69 66 70 70 64 61 52 48 59 67 66 74 83 78
75 84 81 79 79 65 81 84 83 90 90 93 88 72 72 70 78 87 91 87 91 88
83 78 79 77 81 76 78 75 82 84 86 80 80 84 84 81 85 73 72 68 65 79 81
80 76 59 55 66 64 62 67 74 74 70 77 83 81 73 87 89 92 92 93 93 86
76 81 78 73 80 81 52 63 72 75 55 61 71 71 72 75 65 61 60 64 61 63
62 59 56 59 72 71 75 71 71 73 78 73 71 73 71 66 69 69 64 65 54 62 56
55 58 48 54 57 60 61 63 65 71 70 68 69 59 49 39 50 46 54 60 62 65
66 66 63 62 60 63 60 58 34 36 44 53 52 53 58 55 50 48 51 62 40 44 
49 53 53 53 55 51 54 48 53 49 48 51 47 50 51 51 50 39))
  (let* ((sel (if n (select (order (uniform-rand 330)) (iseq n))
                  (iseq 330)))
         (p (plot-points (select temperature sel)
                        (select ozone sel)
                        :variable-labels 
                      '("Temperature" "Ozone") 
                      :title "Temperature vs Ozone Conc" :show (showplot))))
      (send p :plot-controls)
      (send p :show-window)
  p))

(2d-demos)
