;;;; Copyright (C) 1994, 1996 R. Dennis Cook and Sanford Weisberg.  
;;;; Computer code to accompany the book \"Introduction to
;;;; Regression Graphics\", New York:  Wiley, 1994, ISBN 0-471-00839-7. 

(defun pred-demos ()
   (let* ((menu (send menu-proto :new "TranPred"))
          (items '( "Predictors Uncorrelated"
                    "Predictors Linearly Related" 
                    "Predictors Nonlinearly Related" 
                    "Quit"))
          (actions (list #'cprcase1 #'cprcase2 #'cprcase3 
                   #'(lambda () (send menu :remove)))))
   (apply #'send menu :append-items
          (mapcar #'(lambda (i a) (send menu-item-proto :new i :action a))
            items actions))
   (send menu :install)
   menu))

(setf *random-state* (make-random-state
'#(737236956 885348671 1933221075 325426903 654050955 238275119 548504323 
            813576519 1719119931 2114853407 429087795 156842679 772080875 
            1204762895 1745377379 1872964391 1465889947 1232148223 2147483647 
            65536 5 17) ))

(def *z1* (normal-rand 100))
(def *z2* (normal-rand 100))
(def *z3* (uniform-rand 100))
(def *e* (* .5 (normal-rand 100)))

(defun cprcase1 () (case1))
(defun cprcase2 () (case2))
(defun cprcase3 () (case3))

(defun g (z) (^ z -0.67))

(defun case1 ()
  (let* ((z3 (+ .05 (* .95 *z3*)))
         (z1 *z1*)
         (z2 *z2*)
         (e (* .5 *e*))
         (gz (g z3))
         (y (+ 1 (* 1 z1) (* 1 z2) gz)))
        (rcode :data (list z1 z2 z3 (+ y e) gz y e)
               :data-names '("x11" "x12" "x2" "y" "t[x2]" "E[y|x]" "e")
               :info '("N(0,1)" "N(0,1)" "Uniform on 0.05 to 1" "E[y|x]+e" 
                       "x2^{-0.67}" "1 + x11 + x12 + x2^{-0.67}" "N(0,0.25)")
               :description
"Transformation demo discussed in R. D. Cook and S. Weisberg (1994), \"An 
Introduction to Regression Graphics,\" New York:  Wiley, Sec. 9.2.1, p. 139."
               :name "Uncorr")))

(defun case2 ()
  (let* ((z3 (+ .05 (* .95 *z3*)))
         (z1 (+ (* 3 z3) (* 0.57 *z1*)))
         (z2 (+ (* 3 z3) (* 0.57 *z2*)))
         (e (* .5 *e*))
         (gz (g z3))
         (y (+ 1 (* 1 z1) (* 1 z2) gz)))
        (rcode :data (list z1 z2 z3 (+ y e) gz y e)
               :data-names '("x11" "x12" "x2" "y" "t[x2]" "E[y|x]" "e")
               :info '("3*x2 + 0.57*N(0,1)" "3*x2 + 0.57*N(0,1)" 
                       "Uniform on 0.05 to 1" "E[y|x]+e" 
                       "x2^{-0.67}" "1 + x11 + x12 + x2^{-0.67}" "N(0,0.25)")
               :description
"Transformation demo discussed in R. D. Cook and S. Weisberg (1994), \"An 
Introduction to Regression Graphics,\" New York:  Wiley, Sec. 9.2.3, p. 142, 
and Exercise 9.2."
               :name "LinRel")))

(defun case3 ()
  (let* ((z3 (+ .05 (* .95 *z3*)))
         (z1 (+ (abs (- z3 .5)))); (* 1 *z1*)))
         (z2 *z2*)
         (e (* .5 *e*))
         (gz (g z3))
         (y (+ 1 (* 1 z1) (* 1 z2) gz)))
        (rcode :data (list z1 z2 z3 (+ y e) gz y e)
               :data-names '("x11" "x12" "x2" "y" "t[x2]" "E[y|x]" "e")
               :info '("|x2 - 0.5|" "N(0,1)" 
                       "Uniform on 0.05 to 1" "E[y|x]+e" 
                       "x2^{-0.67}" "1 + x11 + x12 + x2^{-0.67}" "N(0,0.25)")
               :description
"Transformation demo discussed in R. D. Cook and S. Weisberg (1994), \"An 
Introduction to Regression Graphics,\" New York:  Wiley, Sec. 9.2.4, p. 142."
               :name "NonlinRel")))

(pred-demos)
